import random

class Field():
    def __init__(self,clear_place_sign = '',player_sign = 'x',bot_sign = 'o',turn = random.choice(['player','bot']) ):
        self.map = [[clear_place_sign,clear_place_sign,clear_place_sign],[clear_place_sign,clear_place_sign,clear_place_sign],[clear_place_sign,clear_place_sign,clear_place_sign]]
        self.clear_place_sign = clear_place_sign
        self.player_sign = player_sign
        self.bot_sign = bot_sign
        self.turn = turn

        self.combinations = [[[0,0],[0,1],[0,2]],[[1,0],[1,1],[1,2]],[[2,0],[2,1],[2,2]],[[0,0],[1,0],[2,0]],[[0,1],[1,1],[2,1]],[[0,2],[1,2],[2,2]],[[0,0],[1,1],[2,2]],[[2,0],[1,1],[0,2]]]

    def get_map(self):
        return self.map
    
    def is_payer_turn(self):
        return self.turn == 'player'

    def make_move(self,position,skip_check = False,place_anyway = False):
        is_all = True
        for i in self.combinations:
            for b in i:
                if self.map[b[0]][b[1]] == self.clear_place_sign:
                    is_all = False
        if is_all:
            return 'draw'

        if skip_check or self.turn == 'player':
            if self.map[position[0]][position[1]] != self.clear_place_sign and not place_anyway:
                raise Exception('Place already taken, if you want to place sign anyway, add `place_anyway` argument with value `True`')
            self.map[position[0]][position[1]] = self.player_sign

        else:
            raise Exception('Not player turn, if you want to make move anyway, add `skip_check` argument with value `True`')

        if not skip_check:        
            self.turn = 'bot'
        return self.check_win()

    def check_win(self):
        is_all = True
        for i in self.combinations:
            for b in i:
                if self.map[b[0]][b[1]] == self.clear_place_sign:
                    is_all = False
        if is_all:
            return 'draw'
            
        for i in self.combinations:
            if self.map[i[0][0]][i[0][1]] != self.clear_place_sign and self.map[i[0][0]][i[0][1]] == self.map[i[1][0]][i[1][1]] == self.map[i[2][0]][i[2][1]]:
                return ('player' if self.map[i[0][0]][i[0][1]] == self.player_sign else 'bot')

        return False

class Bot():
    def __init__(self,field : Field,level = 100):
        if not isinstance(field,Field):
            raise Exception('field should be instance of `Field` class')
        self.field = field
        self.level = 100-level
    
    def make_move(self,skip_check = False):
        def set_new(position):
            self.field.map[position[0]][position[1]] = self.field.bot_sign
            self.field.turn = 'player'
            return self.field.check_win()

        if not skip_check and self.field.turn != 'bot':
            raise Exception('Not bot`s turn. If you want bot to make move anyway, add `skip_check` argument with value `True`')

        is_all = True
        for i in self.field.combinations:
            for b in i:
                if self.field.map[b[0]][b[1]] == self.field.clear_place_sign:
                    is_all = False
        if is_all:
            return 'draw'

        if random.randint(0,100) > self.level:
            # finding place where we win
            for i in self.field.combinations:
                counter = 0
                pos = []
                place = True
                for b in i:
                    if self.field.map[b[0]][b[1]] == self.field.player_sign:
                        place = False
                        break
                    elif self.field.map[b[0]][b[1]] == self.field.clear_place_sign:
                        counter +=1
                        pos = b
                    if counter > 1:
                        place = False
                if place:
                    return set_new(pos)

            #find places where we will lose if dont place
            for i in self.field.combinations:
                counter = 0
                pos = []
                place = True
                for b in i:
                    if self.field.map[b[0]][b[1]] == self.field.bot_sign:
                        place = False
                        break
                    elif self.field.map[b[0]][b[1]] == self.field.clear_place_sign:
                        counter +=1
                        pos = b
                    if counter > 1:
                        place = False
                if place:
                    return set_new(b)
            
        #random position
        positions = []
        for i in self.field.combinations:
            for b in i:
                if self.field.map[b[0]][b[1]] == self.field.clear_place_sign:
                    positions.append(b)
        return set_new(random.choice(positions))


from setuptools import setup, find_packages


l = """[GitHub:](https://github.com/ruslan-ilesik/tic-tac-toe-bot-pip-package)
Usage Example
```python
import tic_tac_toe

map = tic_tac_toe.Field(clear_place_sign= '',player_sign='x',bot_sign='o',turn= 'player') #default values (turn is randomed if defaut, posible ['bot','player'])
bot = tic_tac_toe.Bot(map,level=100) #
print(map.is_payer_turn()) #>> True
print(map.get_map()) #>> [['', '', ''], ['', '', ''], ['', '', '']]
print(map.make_move(position = [0,0],skip_check=False,place_anyway=False)) #position = [0-2,0-2],skip_check - if True, skips check for turn, place_anyway - skip check if place free
#>> False # it prints if someone won after that (could be "bot","player","draw" (when no places free and anyone win),False)
print(map.get_map())#>>[['x', '', ''], ['', '', ''], ['', '', '']]
print(map.is_payer_turn())# >> False
print(bot.make_move())# >> False
print(map.get_map())#>> [['x', 'o', ''], ['', '', ''], ['', '', '']]
print(map.is_payer_turn())# >> True
print(map.make_move(position = [0,1],skip_check=False,place_anyway=True)) # False
print(map.get_map())#>> [['x', 'x, ''], ['', '', ''], ['', '', '']]
print(map.is_payer_turn())# >> False
print(map.make_move(position = [0,2],skip_check=True,place_anyway=False)) # player
print(map.check_win()) #player
```"""


setup(name='tic_tac_toe_api_bot',
      version='0.2.6.1',
      description='simple bot for tic tac toe',
      long_description=l,
      long_description_content_type='text/markdown',
      classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
      ],
      keywords='tic_tac_toe bot api game_api ',
      url='https://github.com/ruslan-ilesik/tic-tac-toe-bot-pip-package',
      author='lesikr',
      license='MIT',
      packages=['tic_tac_toe'],
      install_requires=[],
      include_package_data=True,
      zip_safe=False,
      )